#include "csvreader.h"
#include "utils.h"

#include <fstream>

#include <csv.hpp>

#ifdef DELETE
#undef DELETE
#endif

namespace fs = std::filesystem;

std::size_t CsvReader::readSellersFromFile(const fs::path &filePath, Marketplace *market)
{
    csv::CSVFormat format;
    format.delimiter(';');

#if defined(_WIN64) || defined(_WIN32)
    // Windows: Somhow this is necessary in order to open file names with umlauts
    auto wide = filePath.wstring();
    std::string fileName(wide.begin(), wide.end());

    csv::CSVReader csvReader(fileName, format);
#else
    csv::CSVReader csvReader(filePath.string(), format);
#endif

    for (auto &seller : market->getSellers()) {
        seller->setState(Seller::State::DELETE);
    }

    market->storeToDb(true);

    for (csv::CSVRow &row : csvReader) {
        if (!row[0].is_int()) {
            continue;
        }

        auto seller = std::make_unique<Seller>();
        seller->setSellerNo(row[0].get<int>());
        if (row[1].is_int()) {
            seller->setNumArticlesOffered(row[1].get<int>());
        } else {
            seller->setNumArticlesOffered(0);
        }

        // If both, first name and last name, are empty, use N. N.
        // Else, use the real values.
        if (row[2].get<std::string>().empty() && row[3].get<std::string>().empty()) {
            seller->setFirstName("N.");
            seller->setLastName("N.");
        } else {
            std::string firstName = row[2].get<std::string>();
            seller->setFirstName(trim(firstName));
            std::string lastName = row[3].get<std::string>();
            seller->setLastName(trim(lastName));
        }

        market->getSellers().push_back(std::move(seller));
    }

    // Add one additional seller "RESERVE RESERVE"
    auto seller = std::make_unique<Seller>();
    seller->setSellerNo(market->getNextSellerNo());
    seller->setFirstName("RESERVE");
    seller->setLastName("RESERVE");
    market->getSellers().push_back(std::move(seller));

    // If there was no special seller "Sonderkonto" in import data, then create one
    auto specialSeller = market->findSellerWithSellerNo(0);
    if (!specialSeller) {
        auto seller = std::make_unique<Seller>();
        seller->setSellerNo(0);
        seller->setLastName("Sonderkonto");
        seller->setFirstName("Sonderkonto");
        seller->setNumArticlesOffered(0);
        market->getSellers().push_back(std::move(seller));
    }

    market->sortSellers();
    market->storeToDb();

    return market->getSellers().size() - 1; // minus 1 because we don't count the "special" seller
}
